# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
from .open_text_application_security_credential import (
    OpenTextApplicationSecurityCredential,
)
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class AppsecOpenTextApplicationSecurity(UncheckedBaseModel):
    """
    Configuration for OpenText Application Security (formerly Fortify Software Security Center) as an application security provider.

    **This provider is for on-premise installations of OpenText Application Security. For the SaaS product, please use the OpenText Core Application Security provider.**

    As this provider is for an on-premise only product, either a Synqly Bridge Agent must be used or the OpenText Application Security installation must be accessible from a public URL.

    [Configuration guide](https://docs.synqly.com/guides/provider-configuration/opentext-applicationsecurity-appsec-setup)
    """

    credential: OpenTextApplicationSecurityCredential = pydantic.Field()
    """
    Credentials used for accessing the OpenText Application Security API.
    """

    url: str = pydantic.Field()
    """
    Base URL for the OpenText Application Security API. This URL should
    be the same as the URL used to access the OpenText Application
    Security web interface.
    
    **If a Synqly Bridge Agent is not used, this URL must be a publically accessible**
    
    For more information on using a Synqly Bridge Agent, see the [Synqly Bridge Agent documentation](https://docs.synqly.com/bridge/agent-setup).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ListProviderCapabilitiesExpandOptions(str, enum.Enum):
    CONNECTOR = "connector"
    OPERATIONS = "operations"
    PROVIDER_CONFIG = "provider_config"
    _UNKNOWN = "__LISTPROVIDERCAPABILITIESEXPANDOPTIONS_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "ListProviderCapabilitiesExpandOptions":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        connector: typing.Callable[[], T_Result],
        operations: typing.Callable[[], T_Result],
        provider_config: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is ListProviderCapabilitiesExpandOptions.CONNECTOR:
            return connector()
        if self is ListProviderCapabilitiesExpandOptions.OPERATIONS:
            return operations()
        if self is ListProviderCapabilitiesExpandOptions.PROVIDER_CONFIG:
            return provider_config()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from .types.query_devices_response import QueryDevicesResponse
from ..core.unchecked_base_model import construct_type
from ..common.errors.bad_request_error import BadRequestError
from ..common.types.problem import Problem
from ..common.errors.unauthorized_error import UnauthorizedError
from ..common.errors.forbidden_error import ForbiddenError
from ..common.errors.not_found_error import NotFoundError
from ..common.errors.method_not_allowed_error import MethodNotAllowedError
from ..common.errors.conflict_error import ConflictError
from ..common.errors.unsupported_media_type_error import UnsupportedMediaTypeError
from ..common.errors.too_many_requests_error import TooManyRequestsError
from ..common.errors.internal_server_error import InternalServerError
from ..common.errors.not_implemented_error import NotImplementedError
from ..common.errors.bad_gateway_error import BadGatewayError
from ..common.errors.service_unavailable_error import ServiceUnavailableError
from ..common.errors.gateway_timeout_error import GatewayTimeoutError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from .types.device import Device
from .types.create_device_response import CreateDeviceResponse
from .types.get_labels_response import GetLabelsResponse
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class AssetsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def query_devices(
        self,
        *,
        meta: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        filter: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        order: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> QueryDevicesResponse:
        """
        Query devices from an asset inventory system

        Parameters
        ----------
        meta : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Add metadata to the response by invoking meta functions. Documentation for meta functions is available at https://docs.synqly.com/api-reference/meta-functions. Not all meta function are available at every endpoint.

        limit : typing.Optional[int]
            Number of finding reports to return. Defaults to 50.

        cursor : typing.Optional[str]
            Start search from cursor position.

        filter : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter results by this query. For more information on filtering, refer to the Assets Filtering Guide. Defaults to no filter. If used more than once, the queries are ANDed together.

        order : typing.Optional[str]
            Select a field to order the results by. Defaults to `time`. To control the direction of the sorting, append
            `[asc]` or `[desc]` to the field name. For example, `time[asc]` will sort the results by `time` in ascending order.
            The ordering defaults to `asc` if not specified.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        QueryDevicesResponse

        Examples
        --------
        from synqly import SynqlyEngine

        client = SynqlyEngine(
            token="YOUR_TOKEN",
        )
        client.assets.query_devices(
            meta="string",
            limit=1,
            cursor="string",
            filter="string",
            order="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/assets/devices",
            method="GET",
            params={
                "meta": meta,
                "limit": limit,
                "cursor": cursor,
                "filter": filter,
                "order": order,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    QueryDevicesResponse,
                    construct_type(
                        type_=QueryDevicesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_asset(
        self, *, device: Device, request_options: typing.Optional[RequestOptions] = None
    ) -> CreateDeviceResponse:
        """
        Creates a `Device` object in the token-linked Integration.

        Parameters
        ----------
        device : Device

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateDeviceResponse

        Examples
        --------
        import datetime

        from synqly import SynqlyEngine
        from synqly.ocsf.v_1_3_0.inventoryinfo.classes import InventoryInfo
        from synqly.ocsf.v_1_3_0.inventoryinfo.objects import (
            Account,
            Actor,
            Agent,
            Api,
            Cloud,
            Container,
            Device,
            DeviceHwInfo,
            Display,
            Enrichment,
            Extension,
            Fingerprint,
            Group,
            Idp,
            Image,
            KeyboardInfo,
            Location,
            Logger,
            Metadata,
            NetworkInterface,
            Observable,
            Organization,
            Os,
            Osint,
            Process,
            Product,
            Request,
            Response,
            Service,
            Session,
            User,
        )

        client = SynqlyEngine(
            token="YOUR_TOKEN",
        )
        client.assets.create_asset(
            device=InventoryInfo(
                activity_id=1,
                activity_name="string",
                actor=Actor(
                    actor_type="string",
                    actor_type_id=1,
                    app_name="string",
                    app_uid="string",
                    authorizations=[],
                    groups=[],
                    idp=Idp(),
                    invoked_by="string",
                    process=Process(),
                    session=Session(),
                    user=User(),
                ),
                api=Api(
                    group=Group(),
                    operation="string",
                    request=Request(
                        uid="string",
                    ),
                    response=Response(),
                    service=Service(),
                    version="string",
                ),
                category_name="string",
                category_uid=1,
                class_uid=1,
                cloud=Cloud(
                    account=Account(),
                    org=Organization(),
                    project_uid="string",
                    provider="string",
                    region="string",
                    zone="string",
                ),
                count=1,
                custom_fields={"string": {"key": "value"}},
                device=Device(
                    agent_list=[Agent()],
                    autoscale_uid="string",
                    boot_time=1,
                    boot_time_dt=datetime.datetime.fromisoformat(
                        "2024-01-15 09:30:00+00:00",
                    ),
                    container=Container(
                        hash=Fingerprint(
                            algorithm_id=1,
                            value="string",
                        ),
                        image=Image(
                            uid="string",
                        ),
                        name="string",
                        network_driver="string",
                        orchestrator="string",
                        pod_uuid="string",
                        runtime="string",
                        size=1,
                        tag="string",
                        uid="string",
                    ),
                    created_time=1,
                    created_time_dt=datetime.datetime.fromisoformat(
                        "2024-01-15 09:30:00+00:00",
                    ),
                    desc="string",
                    domain="string",
                    first_seen_time=1,
                    first_seen_time_dt=datetime.datetime.fromisoformat(
                        "2024-01-15 09:30:00+00:00",
                    ),
                    groups=[Group()],
                    hostname="string",
                    hw_info=DeviceHwInfo(
                        bios_date="string",
                        bios_manufacturer="string",
                        bios_uid="string",
                        bios_ver="string",
                        chassis="string",
                        cpu_bits=1,
                        cpu_cores=1,
                        cpu_count=1,
                        cpu_speed=1,
                        cpu_type="string",
                        desktop_display=Display(),
                        keyboard_info=KeyboardInfo(),
                        ram_size=1,
                        serial_number="string",
                    ),
                    hypervisor="string",
                    image=Image(
                        uid="string",
                    ),
                    imei="string",
                    instance_uid="string",
                    interface_name="string",
                    interface_uid="string",
                    ip="string",
                    ip_addresses=["string"],
                    is_compliant=True,
                    is_managed=True,
                    is_personal=True,
                    is_trusted=True,
                    last_seen_time=1,
                    last_seen_time_dt=datetime.datetime.fromisoformat(
                        "2024-01-15 09:30:00+00:00",
                    ),
                    location=Location(
                        city="string",
                        continent="string",
                        coordinates=[],
                        country="string",
                        desc="string",
                        geohash="string",
                        is_on_premises=True,
                        isp="string",
                        lat=1.1,
                        long_=1.1,
                        postal_code="string",
                        provider="string",
                        region="string",
                        timezone="string",
                    ),
                    mac="string",
                    mac_addresses=["string"],
                    modified_time=1,
                    modified_time_dt=datetime.datetime.fromisoformat(
                        "2024-01-15 09:30:00+00:00",
                    ),
                    name="string",
                    namespace_pid=1,
                    netbios_names=["string"],
                    network_interfaces=[
                        NetworkInterface(
                            type_id=1,
                        )
                    ],
                    network_status="string",
                    network_status_id=1,
                    org=Organization(),
                    os=Os(
                        build="string",
                        country="string",
                        cpe_name="string",
                        cpu_bits=1,
                        edition="string",
                        lang="string",
                        name="string",
                        sp_name="string",
                        sp_ver=1,
                        type="string",
                        type_id=1,
                        version="string",
                    ),
                    owner=User(),
                    region="string",
                    risk_level="string",
                    risk_level_id=1,
                    risk_score=1,
                    subnet="string",
                    subnet_uid="string",
                    sw_info=[Product()],
                    type="string",
                    type_id=1,
                    uid="string",
                    uid_alt="string",
                    vendor=Organization(),
                    vlan_uid="string",
                    vpc_uid="string",
                    zone="string",
                ),
                duration=1,
                end_time=1,
                end_time_dt=datetime.datetime.fromisoformat(
                    "2024-01-15 09:30:00+00:00",
                ),
                enrichments=[
                    Enrichment(
                        data={"key": "value"},
                        name="string",
                        value="string",
                    )
                ],
                message="string",
                metadata=Metadata(
                    correlation_uid="string",
                    event_code="string",
                    extension=Extension(
                        name="string",
                        uid="string",
                        version="string",
                    ),
                    extensions=[
                        Extension(
                            name="string",
                            uid="string",
                            version="string",
                        )
                    ],
                    labels=["string"],
                    log_level="string",
                    log_name="string",
                    log_provider="string",
                    log_version="string",
                    logged_time=1,
                    logged_time_dt=datetime.datetime.fromisoformat(
                        "2024-01-15 09:30:00+00:00",
                    ),
                    loggers=[Logger()],
                    modified_time=1,
                    modified_time_dt=datetime.datetime.fromisoformat(
                        "2024-01-15 09:30:00+00:00",
                    ),
                    original_time="string",
                    processed_time=1,
                    processed_time_dt=datetime.datetime.fromisoformat(
                        "2024-01-15 09:30:00+00:00",
                    ),
                    product=Product(),
                    profiles=["string"],
                    sequence=1,
                    tenant_uid="string",
                    uid="string",
                    version="string",
                ),
                observables=[
                    Observable(
                        name="string",
                        type_id=1,
                    )
                ],
                osint=[
                    Osint(
                        type_id=1,
                        value="string",
                    )
                ],
                raw_data="string",
                severity="string",
                severity_id=1,
                start_time=1,
                start_time_dt=datetime.datetime.fromisoformat(
                    "2024-01-15 09:30:00+00:00",
                ),
                status="string",
                status_code="string",
                status_detail="string",
                status_id=1,
                time=1,
                time_dt=datetime.datetime.fromisoformat(
                    "2024-01-15 09:30:00+00:00",
                ),
                timezone_offset=1,
                type_name="string",
                type_uid=1,
                unmapped={"string": {"key": "value"}},
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/assets/devices",
            method="POST",
            json={
                "device": device,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    CreateDeviceResponse,
                    construct_type(
                        type_=CreateDeviceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_labels(
        self,
        *,
        filter: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetLabelsResponse:
        """
        Get labels from an asset inventory system

        Parameters
        ----------
        filter : typing.Optional[str]
            Filter results by this query. For more information on filtering, refer to the Assets Filtering Guide.
            Defaults to no filter.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetLabelsResponse

        Examples
        --------
        from synqly import SynqlyEngine

        client = SynqlyEngine(
            token="YOUR_TOKEN",
        )
        client.assets.get_labels(
            filter="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/assets/labels",
            method="GET",
            params={
                "filter": filter,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetLabelsResponse,
                    construct_type(
                        type_=GetLabelsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncAssetsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def query_devices(
        self,
        *,
        meta: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        filter: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        order: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> QueryDevicesResponse:
        """
        Query devices from an asset inventory system

        Parameters
        ----------
        meta : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Add metadata to the response by invoking meta functions. Documentation for meta functions is available at https://docs.synqly.com/api-reference/meta-functions. Not all meta function are available at every endpoint.

        limit : typing.Optional[int]
            Number of finding reports to return. Defaults to 50.

        cursor : typing.Optional[str]
            Start search from cursor position.

        filter : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter results by this query. For more information on filtering, refer to the Assets Filtering Guide. Defaults to no filter. If used more than once, the queries are ANDed together.

        order : typing.Optional[str]
            Select a field to order the results by. Defaults to `time`. To control the direction of the sorting, append
            `[asc]` or `[desc]` to the field name. For example, `time[asc]` will sort the results by `time` in ascending order.
            The ordering defaults to `asc` if not specified.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        QueryDevicesResponse

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyEngine

        client = AsyncSynqlyEngine(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.assets.query_devices(
                meta="string",
                limit=1,
                cursor="string",
                filter="string",
                order="string",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/assets/devices",
            method="GET",
            params={
                "meta": meta,
                "limit": limit,
                "cursor": cursor,
                "filter": filter,
                "order": order,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    QueryDevicesResponse,
                    construct_type(
                        type_=QueryDevicesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_asset(
        self, *, device: Device, request_options: typing.Optional[RequestOptions] = None
    ) -> CreateDeviceResponse:
        """
        Creates a `Device` object in the token-linked Integration.

        Parameters
        ----------
        device : Device

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateDeviceResponse

        Examples
        --------
        import asyncio
        import datetime

        from synqly import AsyncSynqlyEngine
        from synqly.ocsf.v_1_3_0.inventoryinfo.classes import InventoryInfo
        from synqly.ocsf.v_1_3_0.inventoryinfo.objects import (
            Account,
            Actor,
            Agent,
            Api,
            Cloud,
            Container,
            Device,
            DeviceHwInfo,
            Display,
            Enrichment,
            Extension,
            Fingerprint,
            Group,
            Idp,
            Image,
            KeyboardInfo,
            Location,
            Logger,
            Metadata,
            NetworkInterface,
            Observable,
            Organization,
            Os,
            Osint,
            Process,
            Product,
            Request,
            Response,
            Service,
            Session,
            User,
        )

        client = AsyncSynqlyEngine(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.assets.create_asset(
                device=InventoryInfo(
                    activity_id=1,
                    activity_name="string",
                    actor=Actor(
                        actor_type="string",
                        actor_type_id=1,
                        app_name="string",
                        app_uid="string",
                        authorizations=[],
                        groups=[],
                        idp=Idp(),
                        invoked_by="string",
                        process=Process(),
                        session=Session(),
                        user=User(),
                    ),
                    api=Api(
                        group=Group(),
                        operation="string",
                        request=Request(
                            uid="string",
                        ),
                        response=Response(),
                        service=Service(),
                        version="string",
                    ),
                    category_name="string",
                    category_uid=1,
                    class_uid=1,
                    cloud=Cloud(
                        account=Account(),
                        org=Organization(),
                        project_uid="string",
                        provider="string",
                        region="string",
                        zone="string",
                    ),
                    count=1,
                    custom_fields={"string": {"key": "value"}},
                    device=Device(
                        agent_list=[Agent()],
                        autoscale_uid="string",
                        boot_time=1,
                        boot_time_dt=datetime.datetime.fromisoformat(
                            "2024-01-15 09:30:00+00:00",
                        ),
                        container=Container(
                            hash=Fingerprint(
                                algorithm_id=1,
                                value="string",
                            ),
                            image=Image(
                                uid="string",
                            ),
                            name="string",
                            network_driver="string",
                            orchestrator="string",
                            pod_uuid="string",
                            runtime="string",
                            size=1,
                            tag="string",
                            uid="string",
                        ),
                        created_time=1,
                        created_time_dt=datetime.datetime.fromisoformat(
                            "2024-01-15 09:30:00+00:00",
                        ),
                        desc="string",
                        domain="string",
                        first_seen_time=1,
                        first_seen_time_dt=datetime.datetime.fromisoformat(
                            "2024-01-15 09:30:00+00:00",
                        ),
                        groups=[Group()],
                        hostname="string",
                        hw_info=DeviceHwInfo(
                            bios_date="string",
                            bios_manufacturer="string",
                            bios_uid="string",
                            bios_ver="string",
                            chassis="string",
                            cpu_bits=1,
                            cpu_cores=1,
                            cpu_count=1,
                            cpu_speed=1,
                            cpu_type="string",
                            desktop_display=Display(),
                            keyboard_info=KeyboardInfo(),
                            ram_size=1,
                            serial_number="string",
                        ),
                        hypervisor="string",
                        image=Image(
                            uid="string",
                        ),
                        imei="string",
                        instance_uid="string",
                        interface_name="string",
                        interface_uid="string",
                        ip="string",
                        ip_addresses=["string"],
                        is_compliant=True,
                        is_managed=True,
                        is_personal=True,
                        is_trusted=True,
                        last_seen_time=1,
                        last_seen_time_dt=datetime.datetime.fromisoformat(
                            "2024-01-15 09:30:00+00:00",
                        ),
                        location=Location(
                            city="string",
                            continent="string",
                            coordinates=[],
                            country="string",
                            desc="string",
                            geohash="string",
                            is_on_premises=True,
                            isp="string",
                            lat=1.1,
                            long_=1.1,
                            postal_code="string",
                            provider="string",
                            region="string",
                            timezone="string",
                        ),
                        mac="string",
                        mac_addresses=["string"],
                        modified_time=1,
                        modified_time_dt=datetime.datetime.fromisoformat(
                            "2024-01-15 09:30:00+00:00",
                        ),
                        name="string",
                        namespace_pid=1,
                        netbios_names=["string"],
                        network_interfaces=[
                            NetworkInterface(
                                type_id=1,
                            )
                        ],
                        network_status="string",
                        network_status_id=1,
                        org=Organization(),
                        os=Os(
                            build="string",
                            country="string",
                            cpe_name="string",
                            cpu_bits=1,
                            edition="string",
                            lang="string",
                            name="string",
                            sp_name="string",
                            sp_ver=1,
                            type="string",
                            type_id=1,
                            version="string",
                        ),
                        owner=User(),
                        region="string",
                        risk_level="string",
                        risk_level_id=1,
                        risk_score=1,
                        subnet="string",
                        subnet_uid="string",
                        sw_info=[Product()],
                        type="string",
                        type_id=1,
                        uid="string",
                        uid_alt="string",
                        vendor=Organization(),
                        vlan_uid="string",
                        vpc_uid="string",
                        zone="string",
                    ),
                    duration=1,
                    end_time=1,
                    end_time_dt=datetime.datetime.fromisoformat(
                        "2024-01-15 09:30:00+00:00",
                    ),
                    enrichments=[
                        Enrichment(
                            data={"key": "value"},
                            name="string",
                            value="string",
                        )
                    ],
                    message="string",
                    metadata=Metadata(
                        correlation_uid="string",
                        event_code="string",
                        extension=Extension(
                            name="string",
                            uid="string",
                            version="string",
                        ),
                        extensions=[
                            Extension(
                                name="string",
                                uid="string",
                                version="string",
                            )
                        ],
                        labels=["string"],
                        log_level="string",
                        log_name="string",
                        log_provider="string",
                        log_version="string",
                        logged_time=1,
                        logged_time_dt=datetime.datetime.fromisoformat(
                            "2024-01-15 09:30:00+00:00",
                        ),
                        loggers=[Logger()],
                        modified_time=1,
                        modified_time_dt=datetime.datetime.fromisoformat(
                            "2024-01-15 09:30:00+00:00",
                        ),
                        original_time="string",
                        processed_time=1,
                        processed_time_dt=datetime.datetime.fromisoformat(
                            "2024-01-15 09:30:00+00:00",
                        ),
                        product=Product(),
                        profiles=["string"],
                        sequence=1,
                        tenant_uid="string",
                        uid="string",
                        version="string",
                    ),
                    observables=[
                        Observable(
                            name="string",
                            type_id=1,
                        )
                    ],
                    osint=[
                        Osint(
                            type_id=1,
                            value="string",
                        )
                    ],
                    raw_data="string",
                    severity="string",
                    severity_id=1,
                    start_time=1,
                    start_time_dt=datetime.datetime.fromisoformat(
                        "2024-01-15 09:30:00+00:00",
                    ),
                    status="string",
                    status_code="string",
                    status_detail="string",
                    status_id=1,
                    time=1,
                    time_dt=datetime.datetime.fromisoformat(
                        "2024-01-15 09:30:00+00:00",
                    ),
                    timezone_offset=1,
                    type_name="string",
                    type_uid=1,
                    unmapped={"string": {"key": "value"}},
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/assets/devices",
            method="POST",
            json={
                "device": device,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    CreateDeviceResponse,
                    construct_type(
                        type_=CreateDeviceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_labels(
        self,
        *,
        filter: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetLabelsResponse:
        """
        Get labels from an asset inventory system

        Parameters
        ----------
        filter : typing.Optional[str]
            Filter results by this query. For more information on filtering, refer to the Assets Filtering Guide.
            Defaults to no filter.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetLabelsResponse

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyEngine

        client = AsyncSynqlyEngine(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.assets.get_labels(
                filter="string",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/assets/labels",
            method="GET",
            params={
                "filter": filter,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetLabelsResponse,
                    construct_type(
                        type_=GetLabelsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class FilterOperation(str, enum.Enum):
    EQUALS = "eq"
    """
    Filter for exact matches.
    """

    NOT_EQUALS = "neq"
    """
    Filter for non-matches.
    """

    GREATER_THAN = "gt"
    """
    Filter for values greater than the provided value.
    """

    LESS_THAN = "lt"
    """
    Filter for values less than the provided value.
    """

    GREATER_THAN_OR_EQUALS = "gte"
    """
    Filter for values greater than or equal to the provided value.
    """

    LESS_THAN_OR_EQUALS = "lte"
    """
    Filter for values less than or equal to the provided value.
    """

    IN = "in"
    """
    Filter to match values in the provided comma-separated list.
    """

    LIKE = "like"
    """
    Filter to match values that contain the provided value.
    """

    NOT_IN = "not in"
    """
    Filter to match values not in the provided comma-separated list.
    """

    NOT_LIKE = "not like"
    """
    Filter to match values that do not contain the provided value.
    """

    _UNKNOWN = "__FILTEROPERATION_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "FilterOperation":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        equals: typing.Callable[[], T_Result],
        not_equals: typing.Callable[[], T_Result],
        greater_than: typing.Callable[[], T_Result],
        less_than: typing.Callable[[], T_Result],
        greater_than_or_equals: typing.Callable[[], T_Result],
        less_than_or_equals: typing.Callable[[], T_Result],
        in_: typing.Callable[[], T_Result],
        like: typing.Callable[[], T_Result],
        not_in: typing.Callable[[], T_Result],
        not_like: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is FilterOperation.EQUALS:
            return equals()
        if self is FilterOperation.NOT_EQUALS:
            return not_equals()
        if self is FilterOperation.GREATER_THAN:
            return greater_than()
        if self is FilterOperation.LESS_THAN:
            return less_than()
        if self is FilterOperation.GREATER_THAN_OR_EQUALS:
            return greater_than_or_equals()
        if self is FilterOperation.LESS_THAN_OR_EQUALS:
            return less_than_or_equals()
        if self is FilterOperation.IN:
            return in_()
        if self is FilterOperation.LIKE:
            return like()
        if self is FilterOperation.NOT_IN:
            return not_in()
        if self is FilterOperation.NOT_LIKE:
            return not_like()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from .types.list_organization_webhooks_response import ListOrganizationWebhooksResponse
from ..core.unchecked_base_model import construct_type
from ..common.errors.bad_request_error import BadRequestError
from ..common.types.problem import Problem
from ..common.errors.unauthorized_error import UnauthorizedError
from ..common.errors.forbidden_error import ForbiddenError
from ..common.errors.not_found_error import NotFoundError
from ..common.errors.method_not_allowed_error import MethodNotAllowedError
from ..common.errors.conflict_error import ConflictError
from ..common.errors.unsupported_media_type_error import UnsupportedMediaTypeError
from ..common.errors.too_many_requests_error import TooManyRequestsError
from ..common.errors.internal_server_error import InternalServerError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..organization_webhook_base.types.webhook_id import WebhookId
from .types.get_organization_webhook_response import GetOrganizationWebhookResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..organization_webhook_base.types.webhook_filter import WebhookFilter
from .types.organization_webhook_secret import OrganizationWebhookSecret
from ..organization_base.types.environment import Environment
from .types.create_organization_webhook_response import (
    CreateOrganizationWebhookResponse,
)
from ..credentials.types.credential_id import CredentialId
import datetime as dt
from .types.update_organization_webhook_response import (
    UpdateOrganizationWebhookResponse,
)
from ..common.types.patch_operation import PatchOperation
from .types.patch_organization_webhook_response import PatchOrganizationWebhookResponse
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class OrganizationWebhooksClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        limit: typing.Optional[int] = None,
        start_after: typing.Optional[str] = None,
        order: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        filter: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListOrganizationWebhooksResponse:
        """
        List webhooks for the organization

        Parameters
        ----------
        limit : typing.Optional[int]
            Number of `Webhook` objects to return in this page. Defaults to 100.

        start_after : typing.Optional[str]
            Return `Webhook` objects starting after this `name`.

        order : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Select a field to order the results by. Defaults to `name`. To control the direction of the sorting, append
            `[asc]` or `[desc]` to the field name. For example, `name[desc]` will sort the results by `name` in descending order.
            The ordering defaults to `asc` if not specified. May be used multiple times to order by multiple fields, and the
            ordering is applied in the order the fields are specified.

        filter : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter results by this query. For more information on filtering, refer to our Filtering Guide. Defaults to no filter.
            If used more than once, the queries are ANDed together.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListOrganizationWebhooksResponse

        Examples
        --------
        from synqly import SynqlyManagement

        client = SynqlyManagement(
            token="YOUR_TOKEN",
        )
        client.organization_webhooks.list(
            limit=1,
            start_after="string",
            order="string",
            filter="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/organization/webhooks",
            method="GET",
            params={
                "limit": limit,
                "start_after": start_after,
                "order": order,
                "filter": filter,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ListOrganizationWebhooksResponse,
                    construct_type(
                        type_=ListOrganizationWebhooksResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self,
        webhook_id: WebhookId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetOrganizationWebhookResponse:
        """
        Returns the `Webhook` object matching `{webhookId}`.

        Parameters
        ----------
        webhook_id : WebhookId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetOrganizationWebhookResponse

        Examples
        --------
        from synqly import SynqlyManagement

        client = SynqlyManagement(
            token="YOUR_TOKEN",
        )
        client.organization_webhooks.get(
            webhook_id="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/organization/webhooks/{jsonable_encoder(webhook_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetOrganizationWebhookResponse,
                    construct_type(
                        type_=GetOrganizationWebhookResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        filters: typing.Sequence[WebhookFilter],
        url: str,
        secret: OrganizationWebhookSecret,
        name: typing.Optional[str] = OMIT,
        fullname: typing.Optional[str] = OMIT,
        environment: typing.Optional[Environment] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateOrganizationWebhookResponse:
        """
        Create a new webhook for the organization

        Parameters
        ----------
        filters : typing.Sequence[WebhookFilter]
            Specifies which Webhooks to send.

        url : str
            URL that webhooks will be sent to.

        secret : OrganizationWebhookSecret
            Secret used for signing webhooks. This value is used to verify the authenticity of the webhook payload.

        name : typing.Optional[str]
            Human friendly slug for this webhook

        fullname : typing.Optional[str]
            Fullname for this webhook

        environment : typing.Optional[Environment]
            Environment that the webhook is configured for. Only events for accounts associated with this environment will trigger the webhook.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateOrganizationWebhookResponse

        Examples
        --------
        import datetime

        from synqly import SynqlyManagement
        from synqly.organization_base import Environment
        from synqly.organization_webhook_base import WebhookFilter
        from synqly.organization_webhooks import OrganizationWebhookSecret

        client = SynqlyManagement(
            token="YOUR_TOKEN",
        )
        client.organization_webhooks.create(
            name="string",
            fullname="string",
            environment=Environment.TEST,
            filters=[WebhookFilter.ALL],
            url="string",
            secret=OrganizationWebhookSecret(
                value="string",
                expires=datetime.datetime.fromisoformat(
                    "2024-01-15 09:30:00+00:00",
                ),
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/organization/webhooks",
            method="POST",
            json={
                "name": name,
                "fullname": fullname,
                "environment": environment,
                "filters": filters,
                "url": url,
                "secret": secret,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    CreateOrganizationWebhookResponse,
                    construct_type(
                        type_=CreateOrganizationWebhookResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        webhook_id: WebhookId,
        *,
        id: WebhookId,
        fullname: str,
        environment: Environment,
        filters: typing.Sequence[WebhookFilter],
        url: str,
        credential_id: CredentialId,
        name: str,
        created_at: dt.datetime,
        updated_at: dt.datetime,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateOrganizationWebhookResponse:
        """
        Updates the `Webhook` object matching `{webhookId}`. For more information on
        Organizations and Webhooks, refer to our
        [Synqly Overview](https://docs.synqly.com/docs/synqly-overview).

        Parameters
        ----------
        webhook_id : WebhookId

        id : WebhookId

        fullname : str
            Human friendly slug for this webhook

        environment : Environment
            Environment that the webhook is configured for. Only events associated with this environment will trigger the webhook.

        filters : typing.Sequence[WebhookFilter]
            Specifies which Webhooks to send.

        url : str
            URL that webhooks will be sent to

        credential_id : CredentialId
            Credential contain secret

        name : str
            Human-readable name for this resource

        created_at : dt.datetime
            Time object was originally created

        updated_at : dt.datetime
            Last time object was updated

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateOrganizationWebhookResponse

        Examples
        --------
        import datetime

        from synqly import SynqlyManagement
        from synqly.organization_base import Environment
        from synqly.organization_webhook_base import WebhookFilter

        client = SynqlyManagement(
            token="YOUR_TOKEN",
        )
        client.organization_webhooks.update(
            webhook_id="string",
            id="string",
            fullname="string",
            environment=Environment.TEST,
            filters=[WebhookFilter.ALL],
            url="string",
            credential_id="string",
            name="string",
            created_at=datetime.datetime.fromisoformat(
                "2024-01-15 09:30:00+00:00",
            ),
            updated_at=datetime.datetime.fromisoformat(
                "2024-01-15 09:30:00+00:00",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/organization/webhooks/{jsonable_encoder(webhook_id)}",
            method="PUT",
            json={
                "id": id,
                "fullname": fullname,
                "environment": environment,
                "filters": filters,
                "url": url,
                "credential_id": credential_id,
                "name": name,
                "created_at": created_at,
                "updated_at": updated_at,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    UpdateOrganizationWebhookResponse,
                    construct_type(
                        type_=UpdateOrganizationWebhookResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def patch(
        self,
        webhook_id: WebhookId,
        *,
        request: typing.Sequence[PatchOperation],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PatchOrganizationWebhookResponse:
        """
        Patches the `Webhook` object matching `{webhookId}`. For more information on
        Organizations and Webhooks, refer to our
        [Synqly Overview](https://docs.synqly.com/docs/synqly-overview).

        Parameters
        ----------
        webhook_id : WebhookId

        request : typing.Sequence[PatchOperation]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PatchOrganizationWebhookResponse

        Examples
        --------
        from synqly import SynqlyManagement
        from synqly.common import PatchOp, PatchOperation

        client = SynqlyManagement(
            token="YOUR_TOKEN",
        )
        client.organization_webhooks.patch(
            webhook_id="string",
            request=[
                PatchOperation(
                    op=PatchOp.ADD,
                    path="string",
                    from_="string",
                    value={"key": "value"},
                )
            ],
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/organization/webhooks/{jsonable_encoder(webhook_id)}",
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    PatchOrganizationWebhookResponse,
                    construct_type(
                        type_=PatchOrganizationWebhookResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self,
        webhook_id: WebhookId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Delete a webhook for the organization

        Parameters
        ----------
        webhook_id : WebhookId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from synqly import SynqlyManagement

        client = SynqlyManagement(
            token="YOUR_TOKEN",
        )
        client.organization_webhooks.delete(
            webhook_id="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/organization/webhooks/{jsonable_encoder(webhook_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncOrganizationWebhooksClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        limit: typing.Optional[int] = None,
        start_after: typing.Optional[str] = None,
        order: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        filter: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListOrganizationWebhooksResponse:
        """
        List webhooks for the organization

        Parameters
        ----------
        limit : typing.Optional[int]
            Number of `Webhook` objects to return in this page. Defaults to 100.

        start_after : typing.Optional[str]
            Return `Webhook` objects starting after this `name`.

        order : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Select a field to order the results by. Defaults to `name`. To control the direction of the sorting, append
            `[asc]` or `[desc]` to the field name. For example, `name[desc]` will sort the results by `name` in descending order.
            The ordering defaults to `asc` if not specified. May be used multiple times to order by multiple fields, and the
            ordering is applied in the order the fields are specified.

        filter : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter results by this query. For more information on filtering, refer to our Filtering Guide. Defaults to no filter.
            If used more than once, the queries are ANDed together.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListOrganizationWebhooksResponse

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyManagement

        client = AsyncSynqlyManagement(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.organization_webhooks.list(
                limit=1,
                start_after="string",
                order="string",
                filter="string",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/organization/webhooks",
            method="GET",
            params={
                "limit": limit,
                "start_after": start_after,
                "order": order,
                "filter": filter,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ListOrganizationWebhooksResponse,
                    construct_type(
                        type_=ListOrganizationWebhooksResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self,
        webhook_id: WebhookId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetOrganizationWebhookResponse:
        """
        Returns the `Webhook` object matching `{webhookId}`.

        Parameters
        ----------
        webhook_id : WebhookId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetOrganizationWebhookResponse

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyManagement

        client = AsyncSynqlyManagement(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.organization_webhooks.get(
                webhook_id="string",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/organization/webhooks/{jsonable_encoder(webhook_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetOrganizationWebhookResponse,
                    construct_type(
                        type_=GetOrganizationWebhookResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        filters: typing.Sequence[WebhookFilter],
        url: str,
        secret: OrganizationWebhookSecret,
        name: typing.Optional[str] = OMIT,
        fullname: typing.Optional[str] = OMIT,
        environment: typing.Optional[Environment] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateOrganizationWebhookResponse:
        """
        Create a new webhook for the organization

        Parameters
        ----------
        filters : typing.Sequence[WebhookFilter]
            Specifies which Webhooks to send.

        url : str
            URL that webhooks will be sent to.

        secret : OrganizationWebhookSecret
            Secret used for signing webhooks. This value is used to verify the authenticity of the webhook payload.

        name : typing.Optional[str]
            Human friendly slug for this webhook

        fullname : typing.Optional[str]
            Fullname for this webhook

        environment : typing.Optional[Environment]
            Environment that the webhook is configured for. Only events for accounts associated with this environment will trigger the webhook.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateOrganizationWebhookResponse

        Examples
        --------
        import asyncio
        import datetime

        from synqly import AsyncSynqlyManagement
        from synqly.organization_base import Environment
        from synqly.organization_webhook_base import WebhookFilter
        from synqly.organization_webhooks import OrganizationWebhookSecret

        client = AsyncSynqlyManagement(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.organization_webhooks.create(
                name="string",
                fullname="string",
                environment=Environment.TEST,
                filters=[WebhookFilter.ALL],
                url="string",
                secret=OrganizationWebhookSecret(
                    value="string",
                    expires=datetime.datetime.fromisoformat(
                        "2024-01-15 09:30:00+00:00",
                    ),
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/organization/webhooks",
            method="POST",
            json={
                "name": name,
                "fullname": fullname,
                "environment": environment,
                "filters": filters,
                "url": url,
                "secret": secret,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    CreateOrganizationWebhookResponse,
                    construct_type(
                        type_=CreateOrganizationWebhookResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        webhook_id: WebhookId,
        *,
        id: WebhookId,
        fullname: str,
        environment: Environment,
        filters: typing.Sequence[WebhookFilter],
        url: str,
        credential_id: CredentialId,
        name: str,
        created_at: dt.datetime,
        updated_at: dt.datetime,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateOrganizationWebhookResponse:
        """
        Updates the `Webhook` object matching `{webhookId}`. For more information on
        Organizations and Webhooks, refer to our
        [Synqly Overview](https://docs.synqly.com/docs/synqly-overview).

        Parameters
        ----------
        webhook_id : WebhookId

        id : WebhookId

        fullname : str
            Human friendly slug for this webhook

        environment : Environment
            Environment that the webhook is configured for. Only events associated with this environment will trigger the webhook.

        filters : typing.Sequence[WebhookFilter]
            Specifies which Webhooks to send.

        url : str
            URL that webhooks will be sent to

        credential_id : CredentialId
            Credential contain secret

        name : str
            Human-readable name for this resource

        created_at : dt.datetime
            Time object was originally created

        updated_at : dt.datetime
            Last time object was updated

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateOrganizationWebhookResponse

        Examples
        --------
        import asyncio
        import datetime

        from synqly import AsyncSynqlyManagement
        from synqly.organization_base import Environment
        from synqly.organization_webhook_base import WebhookFilter

        client = AsyncSynqlyManagement(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.organization_webhooks.update(
                webhook_id="string",
                id="string",
                fullname="string",
                environment=Environment.TEST,
                filters=[WebhookFilter.ALL],
                url="string",
                credential_id="string",
                name="string",
                created_at=datetime.datetime.fromisoformat(
                    "2024-01-15 09:30:00+00:00",
                ),
                updated_at=datetime.datetime.fromisoformat(
                    "2024-01-15 09:30:00+00:00",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/organization/webhooks/{jsonable_encoder(webhook_id)}",
            method="PUT",
            json={
                "id": id,
                "fullname": fullname,
                "environment": environment,
                "filters": filters,
                "url": url,
                "credential_id": credential_id,
                "name": name,
                "created_at": created_at,
                "updated_at": updated_at,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    UpdateOrganizationWebhookResponse,
                    construct_type(
                        type_=UpdateOrganizationWebhookResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def patch(
        self,
        webhook_id: WebhookId,
        *,
        request: typing.Sequence[PatchOperation],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PatchOrganizationWebhookResponse:
        """
        Patches the `Webhook` object matching `{webhookId}`. For more information on
        Organizations and Webhooks, refer to our
        [Synqly Overview](https://docs.synqly.com/docs/synqly-overview).

        Parameters
        ----------
        webhook_id : WebhookId

        request : typing.Sequence[PatchOperation]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PatchOrganizationWebhookResponse

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyManagement
        from synqly.common import PatchOp, PatchOperation

        client = AsyncSynqlyManagement(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.organization_webhooks.patch(
                webhook_id="string",
                request=[
                    PatchOperation(
                        op=PatchOp.ADD,
                        path="string",
                        from_="string",
                        value={"key": "value"},
                    )
                ],
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/organization/webhooks/{jsonable_encoder(webhook_id)}",
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    PatchOrganizationWebhookResponse,
                    construct_type(
                        type_=PatchOrganizationWebhookResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self,
        webhook_id: WebhookId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Delete a webhook for the organization

        Parameters
        ----------
        webhook_id : WebhookId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyManagement

        client = AsyncSynqlyManagement(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.organization_webhooks.delete(
                webhook_id="string",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/organization/webhooks/{jsonable_encoder(webhook_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class OpenTextCoreApplicationSecurityUrl(str, enum.Enum):
    AMS_FORTIFY_COM = "https://api.ams.fortify.com"
    EMEA_FORTIFY_COM = "https://api.emea.fortify.com"
    APAC_FORTIFY_COM = "https://api.apac.fortify.com"
    SGP_FORTIFY_COM = "https://api.sgp.fortify.com"
    FED_FORTIFYGOV_COM = "https://api.fed.fortifygov.com"
    TRIAL_FORTIFY_COM = "https://api.trial.fortify.com"
    _UNKNOWN = "__OPENTEXTCOREAPPLICATIONSECURITYURL_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "OpenTextCoreApplicationSecurityUrl":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        ams_fortify_com: typing.Callable[[], T_Result],
        emea_fortify_com: typing.Callable[[], T_Result],
        apac_fortify_com: typing.Callable[[], T_Result],
        sgp_fortify_com: typing.Callable[[], T_Result],
        fed_fortifygov_com: typing.Callable[[], T_Result],
        trial_fortify_com: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is OpenTextCoreApplicationSecurityUrl.AMS_FORTIFY_COM:
            return ams_fortify_com()
        if self is OpenTextCoreApplicationSecurityUrl.EMEA_FORTIFY_COM:
            return emea_fortify_com()
        if self is OpenTextCoreApplicationSecurityUrl.APAC_FORTIFY_COM:
            return apac_fortify_com()
        if self is OpenTextCoreApplicationSecurityUrl.SGP_FORTIFY_COM:
            return sgp_fortify_com()
        if self is OpenTextCoreApplicationSecurityUrl.FED_FORTIFYGOV_COM:
            return fed_fortifygov_com()
        if self is OpenTextCoreApplicationSecurityUrl.TRIAL_FORTIFY_COM:
            return trial_fortify_com()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

from .types import (
    Event,
    Event_AccountChange,
    Event_ApiActivity,
    Event_ApplicationLifecycle,
    Event_ApplicationSecurityPostureFinding,
    Event_Authentication,
    Event_AuthorizeSession,
    Event_BaseEvent,
    Event_CloudActivity,
    Event_CloudResourcesInventoryInfo,
    Event_ComplianceFinding,
    Event_DetectionFinding,
    Event_DeviceConfigState,
    Event_DeviceInventoryInfo,
    Event_DnsActivity,
    Event_EmailActivity,
    Event_EntityManagement,
    Event_FileHostingActivity,
    Event_FileSystemActivity,
    Event_GroupManagement,
    Event_HttpActivity,
    Event_IncidentFinding,
    Event_ModuleActivity,
    Event_NetworkActivity,
    Event_ProcessActivity,
    Event_ScanActivity,
    Event_ScheduledJobActivity,
    Event_SecurityFinding,
    Event_SoftwareInfo,
    Event_UserAccessManagement,
    Event_VulnerabilityFinding,
    Event_WebResourceAccessActivity,
)

__all__ = [
    "Event",
    "Event_AccountChange",
    "Event_ApiActivity",
    "Event_ApplicationLifecycle",
    "Event_ApplicationSecurityPostureFinding",
    "Event_Authentication",
    "Event_AuthorizeSession",
    "Event_BaseEvent",
    "Event_CloudActivity",
    "Event_CloudResourcesInventoryInfo",
    "Event_ComplianceFinding",
    "Event_DetectionFinding",
    "Event_DeviceConfigState",
    "Event_DeviceInventoryInfo",
    "Event_DnsActivity",
    "Event_EmailActivity",
    "Event_EntityManagement",
    "Event_FileHostingActivity",
    "Event_FileSystemActivity",
    "Event_GroupManagement",
    "Event_HttpActivity",
    "Event_IncidentFinding",
    "Event_ModuleActivity",
    "Event_NetworkActivity",
    "Event_ProcessActivity",
    "Event_ScanActivity",
    "Event_ScheduledJobActivity",
    "Event_SecurityFinding",
    "Event_SoftwareInfo",
    "Event_UserAccessManagement",
    "Event_VulnerabilityFinding",
    "Event_WebResourceAccessActivity",
]



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
import pydantic
from .scan_frequency_option import ScanFrequencyOption
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ScanSchedule(UncheckedBaseModel):
    time: str = pydantic.Field()
    """
    Time of the day when the scan are repeated. For scans that are executed once, this is the time when the scan was started. This is formatted as `HH:MM:SS`.
    """

    frequency: ScanFrequencyOption = pydantic.Field()
    """
    Periodicity of the scan; for example, weekly, means that the scan will be repeated every `repeat_interval` weeks.
    """

    repeat_interval: int = pydantic.Field()
    """
    Number of days, weeks, months, or years between scans. For example, `1` means that the scan will be repeated once every `frequency` period.
    """

    days: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Days of the week when the scan will be repeated. For example, `["monday", "friday"]` means that the scan will be repeated on Monday and Friday on the schedule defined by `frequency` and `repeat_interval`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .create_custom_request import CreateCustomRequest
from .create_custom_response import CreateCustomResponse
from .create_custom_response_result import CreateCustomResponseResult
from .custom import Custom
from .custom_id import CustomId
from .get_custom_response import GetCustomResponse
from .list_customs_response import ListCustomsResponse
from .patch_custom_response import PatchCustomResponse
from .update_custom_request import UpdateCustomRequest
from .update_custom_response import UpdateCustomResponse

__all__ = [
    "CreateCustomRequest",
    "CreateCustomResponse",
    "CreateCustomResponseResult",
    "Custom",
    "CustomId",
    "GetCustomResponse",
    "ListCustomsResponse",
    "PatchCustomResponse",
    "UpdateCustomRequest",
    "UpdateCustomResponse",
]



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
from .crowd_strike_credential import CrowdStrikeCredential
import pydantic
import typing
from .crowdstrike_hec_credential import CrowdstrikeHecCredential
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class SiemCrowdstrike(UncheckedBaseModel):
    """
    Configuration for CrowdStrike Falcon® Next-Gen SIEM.

    [Configuration guide](https://docs.synqly.com/guides/provider-configuration/crowdstrike-siem-setup)
    """

    credential: CrowdStrikeCredential = pydantic.Field()
    """
    The credential to use for the CrowdStrike Falcon NextGen SIEM tenant.
    """

    hec_credential: typing.Optional[CrowdstrikeHecCredential] = pydantic.Field(
        default=None
    )
    """
    Token credential to use for connecting to the CrowdStrike HEC service. If not provided, sending events to CrowdStrike is disabled.
    """

    hec_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    The generated CrowdStrike HEC URL provided with your token.
    """

    url: typing.Optional[str] = pydantic.Field(default=None)
    """
    Base URL for the CrowdStrike Falcon® Next-Gen SIEM API.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .create_operation_request import CreateOperationRequest
from .create_operation_response import CreateOperationResponse
from .create_operation_response_result import CreateOperationResponseResult
from .get_operation_response import GetOperationResponse
from .operation_webhook_payload import OperationWebhookPayload

__all__ = [
    "CreateOperationRequest",
    "CreateOperationResponse",
    "CreateOperationResponseResult",
    "GetOperationResponse",
    "OperationWebhookPayload",
]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class VulnerabilityStateFilterValue(str, enum.Enum):
    """
    The normalized state identifier of a security finding
    """

    UNKNOWN = "Unknown"
    NEW = "New"
    IN_PROGRESS = "InProgress"
    RESOLVED = "Resolved"
    OTHER = "Other"
    _UNKNOWN = "__VULNERABILITYSTATEFILTERVALUE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "VulnerabilityStateFilterValue":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        unknown: typing.Callable[[], T_Result],
        new: typing.Callable[[], T_Result],
        in_progress: typing.Callable[[], T_Result],
        resolved: typing.Callable[[], T_Result],
        other: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is VulnerabilityStateFilterValue.UNKNOWN:
            return unknown()
        if self is VulnerabilityStateFilterValue.NEW:
            return new()
        if self is VulnerabilityStateFilterValue.IN_PROGRESS:
            return in_progress()
        if self is VulnerabilityStateFilterValue.RESOLVED:
            return resolved()
        if self is VulnerabilityStateFilterValue.OTHER:
            return other()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .fingerprint import Fingerprint
from .software_component_relationship_id import SoftwareComponentRelationshipId
from .software_component_type_id import SoftwareComponentTypeId
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class SoftwareComponent(UncheckedBaseModel):
    """
    The Software Component object describes characteristics of a software component within a software package.
    """

    author: typing.Optional[str] = pydantic.Field(default=None)
    """
    The author(s) who published the software component.
    """

    hash: typing.Optional[Fingerprint] = pydantic.Field(default=None)
    """
    Cryptographic hash to identify the binary instance of a software component.
    """

    license: typing.Optional[str] = pydantic.Field(default=None)
    """
    The software license applied to this component.
    """

    name: str = pydantic.Field()
    """
    The software component name.
    """

    purl: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Package URL (PURL) to identify the software component. This is a URL that uniquely identifies the component, including the component's name, version, and type. The URL is used to locate and retrieve the component's metadata and content.
    """

    related_component: typing.Optional[str] = pydantic.Field(default=None)
    """
    The package URL (PURL) of the component that this software component has a relationship with.
    """

    relationship: typing.Optional[str] = pydantic.Field(default=None)
    """
    The relationship between two software components, normalized to the caption of the <code>relationship_id</code> value. In the case of 'Other', it is defined by the source.
    """

    relationship_id: typing.Optional[SoftwareComponentRelationshipId] = pydantic.Field(
        default=None
    )
    """
    The normalized identifier of the relationship between two software components.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The type of software component, normalized to the caption of the <code>type_id</code> value. In the case of 'Other', it is defined by the source.
    """

    type_id: typing.Optional[SoftwareComponentTypeId] = pydantic.Field(default=None)
    """
    The type of software component.
    """

    version: str = pydantic.Field()
    """
    The software component version.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
from .agent import Agent
import pydantic
from ...base.types.timestamp import Timestamp
import datetime as dt
from .container import Container
from .group import Group
from ...base.types.hostname import Hostname
from .device_hw_info import DeviceHwInfo
from .image import Image
from ...base.types.ip_address import IpAddress
from .location import Location
from ...base.types.mac_address import MacAddress
from .network_interface import NetworkInterface
from .device_network_status_id import DeviceNetworkStatusId
from .organization import Organization
from .os import Os
from .user import User
from .device_risk_level_id import DeviceRiskLevelId
from ...base.types.subnet import Subnet
from .product import Product
from .device_type_id import DeviceTypeId
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Device(UncheckedBaseModel):
    """
    The Device object represents an addressable computer system or host, which is typically connected to a computer network and participates in the transmission or processing of data within the computer network.
    """

    agent_list: typing.Optional[typing.List[Agent]] = pydantic.Field(default=None)
    """
    A list of <code>agent</code> objects associated with a device, endpoint, or resource.
    """

    autoscale_uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the cloud autoscale configuration.
    """

    boot_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The time the system was booted.
    """

    boot_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time the system was booted.
    """

    boot_uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique identifier of the device that changes after every reboot. For example, the value of <code>/proc/sys/kernel/random/boot_id</code> from Linux's procfs.
    """

    container: typing.Optional[Container] = pydantic.Field(default=None)
    """
    The information describing an instance of a container. A container is a prepackaged, portable system image that runs isolated on an existing system using a container runtime like containerd.
    """

    created_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The time when the device was known to have been created.
    """

    created_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when the device was known to have been created.
    """

    desc: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the device, ordinarily as reported by the operating system.
    """

    domain: typing.Optional[str] = pydantic.Field(default=None)
    """
    The network domain where the device resides. For example: <code>work.example.com</code>.
    """

    eid: typing.Optional[str] = pydantic.Field(default=None)
    """
    An Embedded Identity Document, is a unique serial number that identifies an eSIM-enabled device.
    """

    first_seen_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The initial discovery time of the device.
    """

    first_seen_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The initial discovery time of the device.
    """

    groups: typing.Optional[typing.List[Group]] = pydantic.Field(default=None)
    """
    The group names to which the device belongs. For example: <code>["Windows Laptops", "Engineering"]</code>.
    """

    hostname: typing.Optional[Hostname] = pydantic.Field(default=None)
    """
    The device hostname.
    """

    hw_info: typing.Optional[DeviceHwInfo] = pydantic.Field(default=None)
    """
    The endpoint hardware information.
    """

    hypervisor: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the hypervisor running on the device. For example, <code>Xen</code>, <code>VMware</code>, <code>Hyper-V</code>, <code>VirtualBox</code>, etc.
    """

    iccid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Integrated Circuit Card Identification of a mobile device. Typically it is a unique 18 to 22 digit number that identifies a SIM card.
    """

    image: typing.Optional[Image] = pydantic.Field(default=None)
    """
    The image used as a template to run the virtual machine.
    """

    imei: typing.Optional[str] = pydantic.Field(default=None)
    """
    The International Mobile Equipment Identity that is associated with the device.
    """

    imei_list: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The International Mobile Equipment Identity values that are associated with the device.
    """

    instance_uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of a VM instance.
    """

    interface_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the network interface (e.g. eth2).
    """

    interface_uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the network interface.
    """

    ip: typing.Optional[IpAddress] = pydantic.Field(default=None)
    """
    The device IP address, in either IPv4 or IPv6 format.
    """

    ip_addresses: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A list of IP addresses available on the device
    """

    is_backed_up: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether the device or resource has a backup enabled, such as an automated snapshot or a cloud backup. For example, this is indicated by the <code>cloudBackupEnabled</code> value within JAMF Pro mobile devices or the registration of an AWS ARN with the AWS Backup service.
    """

    is_compliant: typing.Optional[bool] = pydantic.Field(default=None)
    """
    The event occurred on a compliant device.
    """

    is_managed: typing.Optional[bool] = pydantic.Field(default=None)
    """
    The event occurred on a managed device.
    """

    is_mobile_account_active: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether the device has an active mobile account. For example, this is indicated by the <code>itunesStoreAccountActive</code> value within JAMF Pro mobile devices.
    """

    is_personal: typing.Optional[bool] = pydantic.Field(default=None)
    """
    The event occurred on a personal device.
    """

    is_shared: typing.Optional[bool] = pydantic.Field(default=None)
    """
    The event occurred on a shared device.
    """

    is_supervised: typing.Optional[bool] = pydantic.Field(default=None)
    """
    The event occurred on a supervised device. Devices that are supervised are typically mobile devices managed by a Mobile Device Management solution and are restricted from specific behaviors such as Apple AirDrop.
    """

    is_trusted: typing.Optional[bool] = pydantic.Field(default=None)
    """
    The event occurred on a trusted device.
    """

    last_seen_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The most recent discovery time of the device.
    """

    last_seen_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The most recent discovery time of the device.
    """

    location: typing.Optional[Location] = pydantic.Field(default=None)
    """
    The geographical location of the device.
    """

    mac: typing.Optional[MacAddress] = pydantic.Field(default=None)
    """
    The Media Access Control (MAC) address of the endpoint.
    """

    mac_addresses: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A list of MAC addresses available on the device
    """

    meid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Mobile Equipment Identifier. It's a unique number that identifies a Code Division Multiple Access (CDMA) mobile device.
    """

    model: typing.Optional[str] = pydantic.Field(default=None)
    """
    The model of the device. For example <code>ThinkPad X1 Carbon</code>.
    """

    modified_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The time when the device was last known to have been modified.
    """

    modified_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when the device was last known to have been modified.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The alternate device name, ordinarily as assigned by an administrator. <p><b>Note:</b> The <b>Name</b> could be any other string that helps to identify the device, such as a phone number; for example <code>310-555-1234</code>.</p>
    """

    namespace_pid: typing.Optional[int] = pydantic.Field(default=None)
    """
    If running under a process namespace (such as in a container), the process identifier within that process namespace.
    """

    netbios_names: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A list of NetBIOS names available on the device
    """

    network_interfaces: typing.Optional[typing.List[NetworkInterface]] = pydantic.Field(
        default=None
    )
    """
    The physical or virtual network interfaces that are associated with the device, one for each unique MAC address/IP address/hostname/name combination.<p><b>Note:</b> The first element of the array is the network information that pertains to the event.</p>
    """

    network_status: typing.Optional[str] = pydantic.Field(default=None)
    """
    The network isolation status of the endpoint
    """

    network_status_id: typing.Optional[DeviceNetworkStatusId] = pydantic.Field(
        default=None
    )
    """
    The network isolation status ID.
    """

    org: typing.Optional[Organization] = pydantic.Field(default=None)
    """
    Organization and org unit related to the device.
    """

    os: typing.Optional[Os] = pydantic.Field(default=None)
    """
    The endpoint operating system.
    """

    os_machine_uuid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The operating system assigned Machine ID. In Windows, this is the value stored at the registry path: <code>HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Cryptography\MachineGuid</code>. In Linux, this is stored in the file: <code>/etc/machine-id</code>.
    """

    owner: typing.Optional[User] = pydantic.Field(default=None)
    """
    The identity of the service or user account that owns the endpoint or was last logged into it.
    """

    region: typing.Optional[str] = pydantic.Field(default=None)
    """
    The region where the virtual machine is located. For example, an AWS Region.
    """

    risk_level: typing.Optional[str] = pydantic.Field(default=None)
    """
    The risk level, normalized to the caption of the risk_level_id value.
    """

    risk_level_id: typing.Optional[DeviceRiskLevelId] = pydantic.Field(default=None)
    """
    The normalized risk level id.
    """

    risk_score: typing.Optional[int] = pydantic.Field(default=None)
    """
    The risk score as reported by the event source.
    """

    subnet: typing.Optional[Subnet] = pydantic.Field(default=None)
    """
    The subnet mask.
    """

    subnet_uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of a virtual subnet.
    """

    sw_info: typing.Optional[typing.List[Product]] = pydantic.Field(default=None)
    """
    The list of software contained on a device
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The device type. For example: <code>unknown</code>, <code>server</code>, <code>desktop</code>, <code>laptop</code>, <code>tablet</code>, <code>mobile</code>, <code>virtual</code>, <code>browser</code>, <code>plc</code>, <code>scada</code>, <code>dcs</code>, <code>cnc</code>, <code>scientific</code>, <code>medical</code>, <code>lighting</code>, <code>energy</code>, <code>transportation</code> <code>other</code>.
    """

    type_id: DeviceTypeId = pydantic.Field()
    """
    The device type ID.
    """

    udid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Apple assigned Unique Device Identifier (UDID). For iOS, iPadOS, tvOS, watchOS and visionOS devices, this is the UDID. For macOS devices, it is the Provisioning UDID. For example: <code>00008020-008D4548007B4F26</code>
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the device. For example the Windows TargetSID or AWS EC2 ARN.
    """

    uid_alt: typing.Optional[str] = pydantic.Field(default=None)
    """
    An alternate unique identifier of the device if any. For example the ActiveDirectory DN.
    """

    vendor: typing.Optional[Organization] = pydantic.Field(default=None)
    """
    The product vendor that created the device.
    """

    vendor_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The vendor for the device. For example <code>Dell</code> or <code>Lenovo</code>.
    """

    vlan_uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Virtual LAN identifier.
    """

    vpc_uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the Virtual Private Cloud (VPC).
    """

    zone: typing.Optional[str] = pydantic.Field(default=None)
    """
    The network zone or LAN segment.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow
